package de.fitko.fitconnect.impl;

import com.google.inject.Inject;
import com.nimbusds.jose.jwk.RSAKey;
import de.fitko.fitconnect.api.domain.auth.OAuthToken;
import de.fitko.fitconnect.api.domain.model.metadata.Metadata;
import de.fitko.fitconnect.api.domain.model.metadata.attachment.Attachment;
import de.fitko.fitconnect.api.domain.model.metadata.data.Data;
import de.fitko.fitconnect.api.domain.model.submission.Submission;
import de.fitko.fitconnect.api.domain.validation.ValidationResult;
import de.fitko.fitconnect.api.exceptions.internal.AuthenticationException;
import de.fitko.fitconnect.api.services.Sender;
import de.fitko.fitconnect.api.services.auth.OAuthService;
import de.fitko.fitconnect.api.services.crypto.CryptoService;
import de.fitko.fitconnect.api.services.metadata.MetadataService;
import de.fitko.fitconnect.api.services.validation.CertificateValidator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.Optional;

public class SubmissionSender implements Sender {

    private static final Logger logger = LoggerFactory.getLogger(SubmissionSender.class);

    private final OAuthService authService;
    private final CertificateValidator certificateValidator;
    private final CryptoService cryptoService;
    private final MetadataService metadataService;

    @Inject
    public SubmissionSender(final OAuthService authService,
                            final CryptoService encryptionService,
                            final CertificateValidator certificateValidator,
                            final MetadataService metadataService) {
        this.authService = authService;
        this.cryptoService = encryptionService;
        this.certificateValidator = certificateValidator;
        this.metadataService = metadataService;
    }


    @Override
    public ValidationResult validatePublicKey(RSAKey publicKey) {
        return certificateValidator.validatePublicKey(publicKey);
    }

    @Override
    public Optional<Data> encryptSubmissionData(RSAKey publicKey, Data data) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public Optional<Attachment> encryptAttachment(RSAKey publicKey, Attachment attachment) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public Optional<OAuthToken> retrieveOAuthToken(String clientId, String clientSecret, String... scope) {
        try {
            final OAuthToken token = authService.authenticate(clientId, clientSecret, scope);
            logger.debug("successfully retrieved token {}", token.getAccessToken());
            return Optional.of(token);
        } catch (AuthenticationException e) {
            logger.error("client could not be authenticated", e.getMessage());
            return Optional.empty();
        }
    }

    @Override
    public Optional<Metadata> createMetadata(Data data, List<Attachment> attachments) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public Optional<Submission> createSubmission(Submission submission) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public Optional<Submission> sendSubmission(String submissionId, Metadata metadata) {
        throw new UnsupportedOperationException("not yet implemented");
    }
}
