package de.fitko.fitconnect.impl.auth;

import de.fitko.fitconnect.api.domain.auth.OAuthToken;
import de.fitko.fitconnect.api.exceptions.internal.AuthenticationException;
import de.fitko.fitconnect.api.services.auth.OAuthService;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;

import static java.util.stream.Collectors.joining;

public class DefaultOAuthService implements OAuthService {

    private final RestTemplate restTemplate;
    private final String tokenUrl;

    public DefaultOAuthService(final RestTemplate restTemplate, final String tokenUrl) {
        this.restTemplate = restTemplate;
        this.tokenUrl = tokenUrl;
    }

    @Override
    public OAuthToken authenticate(String clientId, String clientSecret, String... scope) throws AuthenticationException {
        final String requestBody = buildRequestBody(clientId, clientSecret, scope);
        return performTokenRequestAsync(requestBody);
    }

    private String buildRequestBody(String clientId, String clientSecret, String... scope) {
        var data = new HashMap<String, String>() {{
            put("grant_type", "client_credentials");
            put("client_id", clientId);
            put("client_secret", clientSecret);

        }};

        if(scope.length > 0){
            data.put("scope", String.join(",", scope));
        }

        return data.entrySet().stream().map(e -> e.getKey() + "=" + e.getValue()).collect(joining("&"));
    }

    private OAuthToken performTokenRequestAsync(final String requestBody) throws AuthenticationException {
        final HttpHeaders headers = getHeaders();
        final HttpEntity<String> entity = new HttpEntity<>(requestBody, headers);
        try {
            return restTemplate.exchange(tokenUrl, HttpMethod.POST, entity, OAuthToken.class).getBody();
        } catch (final RestClientException e) {
            throw new AuthenticationException("could not retrieve OAuth token", e);
        }
    }

    private HttpHeaders getHeaders() {
        var headers = new HttpHeaders();
        headers.setAccept(Collections.singletonList(MediaType.APPLICATION_JSON));
        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        headers.setAcceptCharset(List.of(StandardCharsets.UTF_8));
        return headers;
    }

}
